"""
raven.transport
~~~~~~~~~~~~~~~

:copyright: (c) 2010-2012 by the Sentry Team, see AUTHORS for more details.
:license: BSD, see LICENSE for more details.
"""
# TODO: deprecate this namespace and force non-default (sync + threaded) to
# manually import/register transports somehow
from __future__ import absolute_import
import sys

from raven.transport.base import *  # NOQA
from raven.transport.eventlet import *  # NOQA
from raven.transport.exceptions import *  # NOQA
from raven.transport.gevent import *  # NOQA
from raven.transport.http import *  # NOQA
from raven.transport.requests import *  # NOQA
from raven.transport.registry import *  # NOQA
from raven.transport.twisted import *  # NOQA
from raven.transport.threaded import *  # NOQA
from raven.transport.tornado import *  # NOQA

if sys.version_info >= (3, 3):
    from raven.transport.aiohttp import *  # NOQA
